/*
 *  Copyright (c) 2006  Peter Kliem (Peter.Kliem@jaret.de)
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 */
package de.jaret.ppc.wm.model;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collections;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import de.jaret.ppc.wm.WeightMonitor;
/**
 * List of entries. Contains the math.
 * 
 * @author kliem
 * @version $Id: WeightList.java 239 2006-07-31 21:04:23Z olk $
 */
public class WeightList {
    Map _yearMap = new HashMap(5);
    private Date _firstDate;
    private Date _lastDate;
    
    public void registerEntry(WeightEntry entry) {
        Integer year = entry.getYear();
        Map monthMap = (Map)_yearMap.get(year);
        if (monthMap == null) {
            monthMap = new HashMap(17);
            _yearMap.put(year, monthMap);
        }
        Integer month = entry.getMonth();
        Map dayMap = (Map)monthMap.get(month);
        if (dayMap == null) {
            dayMap = new HashMap(53);
            monthMap.put(month, dayMap);
        }
        Integer day = entry.getDay();
        dayMap.put(day, entry);
        checkFirstDate(entry.getDate());
    }
    
    private void checkFirstDate(Date date) {
        if (_firstDate == null || date.before(_firstDate)) {
            _firstDate = date;
        }
        if (_lastDate == null || date.after(_lastDate)) {
            _lastDate = date;
        } 
    }

    public WeightEntry getEntry(int year, int month, int day) {
        Map monthMap = (Map)_yearMap.get(new Integer(year));
        if (monthMap == null) {
            return null;
        }
        Map dayMap = (Map)monthMap.get(new Integer(month));
        if (dayMap == null) {
            return null;
        }
        return (WeightEntry)dayMap.get(new Integer(day));
    }
    
    static Calendar _calendar = GregorianCalendar.getInstance();
    public WeightEntry getEntry(Date date) {
        _calendar.setTime(date);
        int year = _calendar.get(Calendar.YEAR);
        int month = _calendar.get(Calendar.MONTH);
        int day = _calendar.get(Calendar.DAY_OF_MONTH);
        return getEntry(year, month, day);
    }
    
    
    public String save() {
        List entries = getEntryList();
        String error = null;
        
        File f = new File(WeightMonitor.getInstance().getPreferences().getDataFile());
        //System.out.println("Save to '"+f.getAbsolutePath()+"'");
        FileWriter writer = null;
        try {
            writer = new FileWriter(f);

            Iterator it = entries.iterator();
            while (it.hasNext()) {
                WeightEntry we = (WeightEntry) it.next();
                writer.write(we.toLine()+"\n");
            }
        } catch (Exception e) {
            error = "Error writing data to file '"+f.getAbsolutePath()+"' ("+e.getLocalizedMessage()+")";
        }
        finally {
            if (writer != null) {
                try {
                    writer.close();
                } catch (IOException e) {
                    error = (error == null?"":error)+"\nFATAL: could not close writer("+e.getLocalizedMessage()+")";
                }
            }
        }
        return error;
    }
    
    public String load() {
        String error = null;
        
        File f = new File(WeightMonitor.getInstance().getPreferences().getDataFile());
        if (!f.exists()) {
            return "No data to be loaded - possibly first use";
        }
        FileReader reader = null;
        BufferedReader breader = null;
        WeightEntry lastEntry = null;
        try {
            reader = new FileReader(f);
            breader = new BufferedReader(reader);
            while(breader.ready()) {
                String line = breader.readLine();
                WeightEntry we = new WeightEntry(line);
                registerEntry(we);
                calculateTrend(lastEntry, we);
                lastEntry = we;
            }
        } catch (Exception e) {
            error = "Error reading file '"+f.getAbsolutePath()+"' :"+e.getLocalizedMessage();
        } finally {
            if (reader != null) {
                try {
                    reader.close();
                } catch (IOException e) {
                    error = (error == null?"":error)+"\nFATAL: could not close reader("+e.getLocalizedMessage()+")";
                }
            }
        }
        return error;
    }
    
    
    private void calculateTrend(WeightEntry lastEntry, WeightEntry we) {
        double trendNminus1 = lastEntry != null ? lastEntry.getAvgWeigth(): we.getWeight();
        double curTrend = trendNminus1 + 0.1*(we.getWeight()-trendNminus1);
        we.setAvgWeigth(curTrend);
    }

    public List getEntryList() {
        List entries = new ArrayList();
        
        List years = new ArrayList();
        years.addAll(_yearMap.keySet());
        Collections.sort(years);
        
        Iterator yIt = years.iterator();
        while (yIt.hasNext()) {
            Integer year = (Integer) yIt.next();
            Map monthMap = (Map)_yearMap.get(year);
            List months = new ArrayList();
            months.addAll(monthMap.keySet());
            Collections.sort(months);
            Iterator mIt = months.iterator();
            while (mIt.hasNext()) {
                Integer month = (Integer) mIt.next();
                Map dayMap = (Map)monthMap.get(month);
                for (int i=1;i<=31;i++) {
                    WeightEntry we = (WeightEntry)dayMap.get(new Integer(i));
                    if (we != null) {
                        entries.add(we);
                    }
                }
            }
        }
        
        return entries;
    }
    
    public void dump() {
        List entries = getEntryList();
        
        Iterator it = entries.iterator();
        while (it.hasNext()) {
            WeightEntry we = (WeightEntry) it.next();
            System.out.println(we.toLine());
        }
        
    }

    private WeightEntry getPredecessor(WeightEntry we) {
        return getPredecessor(we.getDate());
    }
    private WeightEntry getPredecessor(Date date) {
        WeightEntry result = null;
        Calendar calendar = GregorianCalendar.getInstance();
        calendar.setTime(date);
        calendar.add(Calendar.DAY_OF_MONTH, -1);
        while (!calendar.getTime().before(_firstDate)) {
          //  System.out.println("checking "+calendar.getTime());
            WeightEntry entry = getEntry(calendar.getTime());
            if (entry != null) {
                result = entry;
                break;
            }
            calendar.add(Calendar.DAY_OF_MONTH, -1);
        }
        
        return result;
    }
    
    public void updateTrend(WeightEntry we) {
        WeightEntry predecessor = getPredecessor(we);
        calculateTrend(predecessor, we);
    }

    public List getEntries(Date last, int n) {
        Calendar calendar = GregorianCalendar.getInstance();
        calendar.setTime(last);
        calendar.add(Calendar.DAY_OF_MONTH, -n);
        return getEntriesFirst(calendar.getTime(), n+1);
    }
    public List getEntriesFirst(Date first, int n) {
        List result = new ArrayList(n);

        Calendar calendar = GregorianCalendar.getInstance();
        calendar.setTime(first);
        for (int i=0;i<n;i++) {
            WeightEntry we = getEntry(calendar.getTime());
            if (we == null) {
                WeightEntry pred = getPredecessor(calendar.getTime());
                we = new WeightEntry(calendar.getTime()); // create fake entry
                if (pred != null) { // if pred found -> use the trend value of the predecessor
                    we.setAvgWeigth(pred.getAvgWeigth());
                }
            }
            result.add(we);
            calendar.add(Calendar.DAY_OF_MONTH, 1);
        }
        
        return result;
    }
    
    public class Trend {
        public boolean valid;
        public int days;
        public double kgPerDay;
        public double kgAbs;
        public double calPerDay;
    }
    public Trend getTrend(int days) {
        Trend t = new Trend();
        List entries = getEntries(new Date(), days);
        if (((WeightEntry)entries.get(0)).getAvgWeigth() != 0) {
            t.valid = true;
            t.days = days;
            WeightEntry weFirst = (WeightEntry)entries.get(0);
            WeightEntry weLast = (WeightEntry)entries.get(entries.size()-1);
            t.kgAbs = weLast.getAvgWeigth()-weFirst.getAvgWeigth();
            t.kgPerDay = t.kgAbs/days;
        }
        return t;
    }
    
}
