/*
 *  Copyright (c) 2006  Peter Kliem (Peter.Kliem@jaret.de)
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 */
package de.jaret.ppc.wm.ui;

import java.text.DateFormat;
import java.text.NumberFormat;
import java.text.SimpleDateFormat;
import java.util.Iterator;
import java.util.List;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.PaintEvent;
import org.eclipse.swt.events.PaintListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.widgets.Canvas;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;

import de.jaret.ppc.wm.model.WeightEntry;

/**
 * Simple graph widget.
 * 
 * @author kliem
 * @version $Id: Graph.java 237 2006-07-30 21:16:59Z olk $
 */
public class Graph extends Canvas {
    private static final DateFormat _df = new SimpleDateFormat("dd.MM");
    private static NumberFormat _nf = NumberFormat.getNumberInstance();

    private static final int YAXISPOS = 30;
    private static final int XAXISOFF = 30;
    private static final int RIGHTMARGIN = 3;
    
    private List _entries;
    private double _minWeight = -1;
    private double _maxWeight = -1;


    private int _xAxisPosY;
    private int _xLabelWidth = -1;

    private int _fontHeight = -1;

    private boolean _drawMarks = true;

    public Graph(Composite parent, int style) {
        super(parent, style);
        _nf.setMaximumFractionDigits(1);
        _nf.setMinimumFractionDigits(1);

        addPaintListener(new PaintListener() {
            public void paintControl(PaintEvent e) {
                paint(e);
            }
        });

    }

    public void setData(List entries) {
        _entries = entries;
        _minWeight = -1;
        _maxWeight = -1;
        calcMinMax();
        redraw();
    }

    public void setDrawMark(boolean drawMarks) {
        _drawMarks = drawMarks;
        redraw();
    }

    protected void paint(PaintEvent e) {
        GC gc = e.gc;
        if (_fontHeight == -1) {
            _fontHeight = gc.getFontMetrics().getHeight();
        }
        if (_entries != null) {
            drawXAxis(gc);
            drawYAxis(gc);
            drawValues(gc);
        }
    }

    private void drawValues(GC gc) {
        Iterator it = _entries.iterator();
        Point lastPoint = null;
        int idx = 0;
        while (it.hasNext()) {
            WeightEntry we = (WeightEntry) it.next();
            int x = xForIndex(idx);
            if (we.getAvgWeigth() > 0) {
                int y = yForWeight(we.getAvgWeigth());
                if (lastPoint != null) {
                    gc.drawLine(lastPoint.x, lastPoint.y, x, y);
                }
                lastPoint = new Point(x, y);
            }
            if (we.getWeight() > 0) {
                int y = yForWeight(we.getWeight());
                Color fg = gc.getForeground();
                Color bg = gc.getBackground();
                double diff = we.getAvgWeigth() - we.getWeight();
                Color color;
                if (diff < 0) {
                    color = Display.getCurrent().getSystemColor(SWT.COLOR_RED);
                } else {
                    color = Display.getCurrent().getSystemColor(SWT.COLOR_DARK_GREEN);
                }
                gc.setForeground(color);

                gc.drawLine(lastPoint.x, lastPoint.y, x, y);

                if (_drawMarks) {
                    if (we.isMark()) {
                        gc.setBackground(color);
                        gc.fillOval(x - 2, y - 2, 4, 4);
                    } else {
                        gc.drawOval(x - 2, y - 2, 4, 4);
                    }
                }

                gc.setForeground(fg);
                gc.setBackground(bg);
            }
            idx++;
        }
    }

    private void drawXAxis(GC gc) {
        if (_xLabelWidth == -1) {
            _xLabelWidth = gc.getFontMetrics().getAverageCharWidth() * 5;
        }
        Rectangle r = getClientArea();
        _xAxisPosY = r.y + r.height - XAXISOFF;

        gc.drawLine(YAXISPOS, _xAxisPosY, r.x + r.width, _xAxisPosY);
        double pixPerDay = (double) (r.x + r.width - YAXISPOS-RIGHTMARGIN) / (double) _entries.size();
        int labelMod = (int) Math.ceil(((double) (_xLabelWidth + 10) / pixPerDay));
        double x = YAXISPOS;
        for (int d = 0; d < _entries.size(); d++) {
            if (d % labelMod == 0) {
                WeightEntry we = (WeightEntry) _entries.get(d);
                gc.drawString(_df.format(we.getDate()), (int) x - 4, _xAxisPosY + 5);
                gc.drawLine((int) x, _xAxisPosY, (int) x, _xAxisPosY + 5);
            } else {
                gc.drawLine((int) x, _xAxisPosY, (int) x, _xAxisPosY + 3);
            }
            x += pixPerDay;
        }
    }

    private int xForIndex(int idx) {
        Rectangle r = getClientArea();
        double pixPerDay = (double) (r.x + r.width - YAXISPOS-RIGHTMARGIN) / (double) _entries.size();
        double x = YAXISPOS + ((double) idx) * pixPerDay;
        return (int) x;
    }

    private void drawYAxis(GC gc) {
        Rectangle r = getClientArea();

        gc.drawLine(YAXISPOS, r.y, YAXISPOS, _xAxisPosY);
        double pixPerKg = (double) (_xAxisPosY - r.y) / (double) (_maxWeight - _minWeight);

        double y = r.y;
        int diff = (int) (_maxWeight - _minWeight);

        gc.drawString(_nf.format(_maxWeight), r.x, r.y);

        for (int d = 0; d <= diff; d++) {
            gc.drawLine(YAXISPOS - 4, (int) y, YAXISPOS, (int) y);
            if (d > 0) {
                gc.drawString(_nf.format(_maxWeight - d), r.x, (int) (y - 0.5 * _fontHeight));
            }
            y += pixPerKg;
        }
    }

    private int yForWeight(double weight) {
        Rectangle r = getClientArea();

        double pixPerKg = (double) (_xAxisPosY - r.y) / (double) (_maxWeight - _minWeight);

        double diff = weight - _minWeight;
        return _xAxisPosY - (int) (pixPerKg * diff);
    }

    private void calcMinMax() {
        Iterator it = _entries.iterator();
        while (it.hasNext()) {
            WeightEntry we = (WeightEntry) it.next();
            if (we.getAvgWeigth() > 0) {
                if (_minWeight < 0 || we.getAvgWeigth() < _minWeight) {
                    _minWeight = we.getAvgWeigth();
                }
                if (_maxWeight < 0 || we.getAvgWeigth() > _maxWeight) {
                    _maxWeight = we.getAvgWeigth();
                }
            }
            if (we.getWeight() > 0) {
                if (_minWeight < 0 || we.getWeight() < _minWeight) {
                    _minWeight = we.getWeight();
                }
                if (_maxWeight < 0 || we.getWeight() > _maxWeight) {
                    _maxWeight = we.getWeight();
                }
            }
        }
        _minWeight = Math.floor(_minWeight);
        _maxWeight = Math.ceil(_maxWeight);
    }

}
